/*
 * Support for executing code in new tasks and threads.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <exec/elf.h>
#include <exec/external_pager.h>
#include <exec/memory.h>
#include <exec/process.h>
#include <ipc/map.h>
#include <ipc/server.h>



/* Process creator functionality. */

class ProcessCreating
{
protected:
  const char *_rm_filename;

  /* External pager configuration. */

  ExternalPager _exec_pager;
  ipc_server_config_type _config;

  Process _process;

  /* Stack and payload descriptions. */

  ExplicitSegment _rm_stack;
  Payload *_rm_payload;

  ExplicitSegment _program_stack;
  Payload *_program_payload;

  /* IPC gate for communication within the created task, plus allocated
     capability. */

  l4_cap_idx_t _ipc_gate, _ipc_gate_cap;

  /* Utility methods. */

  long start_pager();

  long init_region_mapper();

  long init_program(file_t *file);

  long init_external_pager();

  long configure_task();

  long create_ipc_gate();

  void init_region(struct exec_region *regions,
                   struct ipc_mapped_cap *mapped_caps,
                   struct exec_region &r, unsigned int &index);

  long start_region_mapper();

  long start_program(int argc, const char *argv[]);

public:
  explicit ProcessCreating(const char *rm_filename);

  virtual long start(file_t *file, int argc, const char *argv[]);

  virtual void set_notifier(Notifier *notifier);
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
